@@ -160,6 +160,10 @@ setup_rt_runtime (runtime_spec_schema_config_linux_resources *resources,
   if (UNLIKELY (ret < 0))
     return ret;
 
+  ret = crun_ensure_directory (cgroup_path, 0755, true, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
   dirfd = open (cgroup_path, O_DIRECTORY | O_CLOEXEC);
   if (UNLIKELY (dirfd < 0))
     return crun_make_error (err, errno, "open `%s`", cgroup_path);
@@ -244,7 +248,11 @@ setup_cpuset_for_systemd_v1 (runtime_spec_schema_config_linux_resources *resourc
   if (UNLIKELY (ret < 0))
     return ret;
 
-  ret = initialize_cpuset_subsystem (cgroup_path, err);
+  ret = crun_ensure_directory (cgroup_path, 0755, true, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  ret = initialize_cpuset_subsystem_resources (cgroup_path, resources, err);
   if (UNLIKELY (ret < 0))
     return ret;
 
@@ -320,6 +328,14 @@ systemd_finalize (struct libcrun_cgroup_args *args, char **path_out,
       if (geteuid ())
         return 0;
 
+      ret = setup_rt_runtime (resources, path, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+
+      ret = setup_cpuset_for_systemd_v1 (resources, path, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+
       for (from = strtok_r (content, "\n", &saveptr); from; from = strtok_r (NULL, "\n", &saveptr))
         {
           char *subpath, *subsystem;
@@ -351,14 +367,6 @@ systemd_finalize (struct libcrun_cgroup_args *args, char **path_out,
             }
         }
 
-      ret = setup_rt_runtime (resources, path, err);
-      if (UNLIKELY (ret < 0))
-        return ret;
-
-      ret = setup_cpuset_for_systemd_v1 (resources, path, err);
-      if (UNLIKELY (ret < 0))
-        return ret;
-
       break;
 
     case CGROUP_MODE_UNIFIED:
@@ -746,7 +754,7 @@ static int
 open_sd_bus_connection (sd_bus **bus, libcrun_error_t *err)
 {
   int rootless;
-  int sd_err;
+  int sd_err = 0;
 
   rootless = is_rootless (err);
   if (UNLIKELY (rootless < 0))
@@ -754,7 +762,7 @@ open_sd_bus_connection (sd_bus **bus, libcrun_error_t *err)
 
   if (rootless)
     sd_err = sd_bus_default_user (bus);
-  else
+  if (! rootless || sd_err < 0)
     sd_err = sd_bus_default_system (bus);
   if (sd_err < 0)
     return crun_make_error (err, -sd_err, "cannot open sd-bus");
